
THLM_FILE_LOOTMOD2_LUA = 1;

-- Increment this to give everyone a message that they need to upgrade
THLM_VERSION = 19;
THLM_VersionWarningShown = false;

THLM_AUCTIONSTART_SUFFIX = " is up, send tells now";
THLM_AUCTION_TIME = 30;

THLM_AUCTIONCANCEL_PREFIX = "The auction for ";
THLM_AUCTIONCANCEL_SUFFIX = " has been cancelled.";

local THLM_ChatFrameHooked = false;
local THLM_Old_ChatFrame_OnEvent = nil;

local THLM_Debug = false;
local THLM_MyPoints = 0;
local THLM_MyPointsKnown = false;
local THLM_MyMinBid = 0;
local THLM_MyMinBidKnown = false;
local THLM_MyMaxBid = 0;
local THLM_MyMaxBidKnown = false;

THLM_AllPoints = {};
THLM_AllMinBids = {};
THLM_AllMaxBids = {};

THLM_CurrentItem = nil;
THLM_NeedIcon = nil;
THLM_NeedIconItemLink = nil;
THLM_CurrentStartTime = 0;
THLM_CurrentExpireTime = 0;
THLM_CurrentAuctioneer = nil;

function THLM_DebugPrint(msg)
    if THLM_Debug then
        THLM_Print(msg);
    end
end

function THLM_Print(msg)
    DEFAULT_CHAT_FRAME:AddMessage("[THLootMod2] " .. msg);
end

function THLM_SetMyPoints(pt)
    THLM_MyPoints = pt;
    THLM_MyPointsKnown = true;

    THLM_AuctionFrameMyPoints:SetText("Points: "..pt);
    THLM_AuctionFrameMyPoints:Show();
end

function THLM_SetMyMinBid(pt)
    THLM_MyMinBid = pt;
    THLM_MyMinBidKnown = true;
    --THLM_DebugPrint("my min bid set to "..pt);
end

function THLM_SetMyMaxBid(pt)
    THLM_MyMaxBid = pt;
    THLM_MyMaxBidKnown = true;
    --THLM_DebugPrint("my max bid set to "..pt);
end

function THLM_SetupChatFrameHook()
    if not THLM_ChatFrameHooked then
        THLM_ChatFrameHooked = true;
        THLM_Old_ChatFrame_OnEvent = ChatFrame_OnEvent;
        ChatFrame_OnEvent = THLM_ChatFrame_OnEvent;
    end
end

function THLM_ChatFrame_OnEvent(event)
    if (event=="CHAT_MSG_WHISPER" and string.find(arg1, "<THLootMod2>", 1, true)) then
		if not THLM_Debug then
        	return;
		end
    end
    if (event=="CHAT_MSG_WHISPER_INFORM" and string.find(arg1, "<THLootMod2>", 1, true)) then
		if not THLM_Debug then
        	return;
		end
    end
    THLM_Old_ChatFrame_OnEvent(event);
end

function THLM_GetCurrentItemHyperlink()
    if (THLM_CurrentItem == nil) then
        return nil;
    end
    local _, _, res = string.find(THLM_CurrentItem,"(item:%d+:%d+:%d+:%d+)");
    return res;
end

StaticPopupDialogs["THLM_PLACE_BID"] = {
        text = "Please enter your bid:",
        button1 = TEXT(ACCEPT),
        button2 = TEXT(CANCEL),
        hasEditBox = 1,
        maxLetters = 4,
        OnAccept = function()
                local text = getglobal(this:GetParent():GetName().."EditBox"):GetText();
                THLM_TryPlaceBid(text);
        end,
        EditBoxOnEnterPressed = function()
                local text = getglobal(this:GetParent():GetName().."EditBox"):GetText();
                THLM_TryPlaceBid(text);
        end,
        OnShow = function()
                getglobal(this:GetName().."EditBox"):SetFocus();
        end,
        OnHide = function()
                if (ChatFrameEditBox:IsVisible()) then
                        ChatFrameEditBox:SetFocus();
                end
                getglobal(this:GetName().."EditBox"):SetText("");
        end,
        OnUpdate = function()
                if (THLM_MyMinBidKnown) then
                        getglobal(this:GetName().."Text"):SetText("Please enter your bid (".. THLM_MyMinBid .." minimum):");
                else
                        getglobal(this:GetName().."Text"):SetText("Please enter your bid:");
                end
        end,
        timeout = 0,
        whileDead = 1,
        exclusive = 1,
        hideOnEscape = 1
};

function THLM_OpenBidWindow()
    StaticPopup_Show("THLM_PLACE_BID", "", "");
end

function THLM_TryPlaceBid(svalue)
    StaticPopup_Hide("THLM_PLACE_BID");
    local value = tonumber(svalue);
    if (value==nil) or (value < 0) then
        THLM_Print("Invalid bid.");
        return;
    end
    if THLM_MyMaxBidKnown and (value > THLM_MyMaxBid) then
		if THLM_MyMinBidKnown and (value == THLM_MyMinBid) then
			-- Always allow someone to bid their minimum
		else
        	THLM_Print("You cannot bid more than "..THLM_MyMaxBid.." point(s).");
        	return;
		end
    end
    if (not THLM_MyMaxBidKnown) and THLM_MyPointsKnown and (value > THLM_MyPoints) then
		if THLM_MyMinBidKnown and (value == THLM_MyMinBid) then
			-- Always allow someone to bid their minimum
		else
        	THLM_Print("You do not have that many points!");
        	return;
		end
    end
    if THLM_MyMinBidKnown and (value < THLM_MyMinBid) and not (value==0) then
        THLM_Print("You must bid at least "..THLM_MyMinBid.." point(s).");
        return;
    end
    THLM_AuctionFrameBidButton:Hide();
    --SendChatMessage("<THLootMod2> I bid "..value.." for "..THLM_CurrentItem, "WHISPER", "COMMON", THLM_CurrentAuctioneer);
    SendChatMessage("bid "..value, "WHISPER", "COMMON", THLM_CurrentAuctioneer);
    THLM_Print("You bid "..value.." for "..THLM_CurrentItem);
end

function THLM_ShowAuctionWindow(item, itemlink, who)
    THLM_AllPoints = {};
    THLM_AllMinBids = {};
    THLM_AllMaxBids = {};
    THLM_MyPointsKnown = false;
    THLM_MyMinBidKnown = false;
    THLM_MyMaxBidKnown = false;
    THLM_CurrentItem = itemlink;
    THLM_CurrentStartTime = GetTime();
    THLM_CurrentExpireTime = THLM_CurrentStartTime + THLM_AUCTION_TIME;
    if (who == UnitName("player")) then
        THLM_CurrentExpireTime = THLM_CurrentExpireTime + 5;
    end
    THLM_CurrentAuctioneer = who;

	THLM_NeedIcon = item;
	THLM_NeedIconItemLink = itemlink;
end

function THLM_PopulateAuctionWindow(item, itemlink)
    local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType,
          itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(item);
    local itemColor;
	local isEpic = nil;
	if ((itemRarity) and (itemRarity >= 4)) then
		isEpic = 1;
	else
		isEpic = nil;
	end
    if (itemName == nil) then
        itemTexture = nil;
        _, _, itemName = string.find(itemlink, "|h%[(.-)%]|h");
        _, _, itemColor = string.find(itemlink, "|c(%x+)|");
        THLM_NeedIcon = item;
        THLM_NeedIconItemLink = itemlink;
    else
        _, _, _, itemColor = GetItemQualityColor(itemRarity);
        _, _, itemColor = string.find(itemColor, "|c(%x+)");
        THLM_NeedIcon = nil;
        THLM_NeedIconItemLink = nil;
    end
    if not (itemName == nil) then
		if (isEpic) then
			THLM_AuctionFrame:SetBackdrop({bgFile = "Interface\\DialogFrame\\UI-DialogBox-Gold-Background", edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Gold-Border", tile = true, tileSize = 32, edgeSize = 32, insets = { left = 11, right = 12, top = 12, bottom = 11 } } );
			THLM_AuctionFrameCorner:SetTexture("Interface\\DialogFrame\\UI-DialogBox-Gold-Corner");
			THLM_AuctionFrameDecoration:Show();
		else
			THLM_AuctionFrame:SetBackdrop({bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background", edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border", tile = true, tileSize = 32, edgeSize = 32, insets = { left = 11, right = 12, top = 12, bottom = 11 } } );
			THLM_AuctionFrameCorner:SetTexture("Interface\\DialogFrame\\UI-DialogBox-Corner");
			THLM_AuctionFrameDecoration:Hide();
		end
	    SetItemButtonTexture(THLM_AuctionFrameIconFrame2, itemTexture);
		THLM_SetUsableColor(THLM_AuctionFrameIconFrame2, item);
		THLM_AuctionFrameIconFrame2.hasItem = 1;
        -- THLM_AuctionFrameIconFrameIcon:SetTexture(itemTexture);
        THLM_AuctionFrameName:SetText("|c" .. itemColor .. itemName);
        THLM_AuctionFrameBidButton:Show();
    	if not THLM_MyPointsKnown then
        	THLM_AuctionFrameMyPoints:Hide();
		end
		THLM_PopulateExtendedInfo(item);
        THLM_AuctionFrame:Show();

        if (type(THLM_DistributePointInfo) == "function") then
            THLM_DistributePointInfo();
        end
    end
end

local THLM_ItemsWithExtendedInfo = {};

function THLM_PopulateExtendedTable()
	local AddItemPair = function(itema, itemb)
		THLM_ItemsWithExtendedInfo["item:" .. itema] = itemb;
		THLM_ItemsWithExtendedInfo["item:" .. itemb] = itema;
	end;
	local AddItemList = function(itema, itemlist)
	end;

    -- == SUNMOTE TRADE ITEMS == --
	AddItemPair(34397, 34211); -- Bladed Chaos Tunic / Harness of Carnal Instinct
	AddItemPair(34394, 34215); -- Breastplate of Agony's Aversion / Warharness of Reckless Fury
	AddItemPair(34384, 34169); -- Breeches of Natural Splendor / Breeches of Natural Aggression
	AddItemPair(34403, 34345); -- Cover of Ursoc the Mighty / Cover of Ursol the Wise
	AddItemPair(34400, 34345); -- Crown of Dath'Remar / Crown of Anasterian
	AddItemPair(34392, 34195); -- Demontooth Shoulderpads / Shoulderpads of Vehemence
	AddItemPair(34390, 34208); -- Erupting Epaulets / Equilibrium Epaulets
	AddItemPair(34381, 34180); -- Felstrength Legplates / Felfury Legplates
	AddItemPair(34396, 34229); -- Garments of Crashing Shores / Garments of Serene Shores
	AddItemPair(34409, 34350); -- Gauntlets of the Ancient Frostwolf / Gauntlets of the Ancient Shadowmoon
	AddItemPair(34408, 34234); -- Gloves of the Forest Drifter / Shadowed Gauntlets of Paroxysm
	AddItemPair(34406, 34342); -- Gloves of Tyri's Power / Handguards of the Dawn
	AddItemPair(34405, 34339); -- Helm of Arcane Purity / Cowl of Light's Purity
	AddItemPair(34401, 34243); -- Helm of Uther's Resolve / Helm of Burning Righteousness
	AddItemPair(34382, 34167); -- Judicator's Legguards / Legplates of the Holy Juggernaut
	AddItemPair(34383, 34186); -- Kilt of Spiritual Reconstruction / Chain Links of the Tumultuous Storm
	AddItemPair(34385, 34188); -- Leggings of the Immortal Beast / Leggings of the Immortal Night
	AddItemPair(34404, 34244); -- Mask of the Fury Hunter / Duplicitous Guise
	AddItemPair(34395, 34216); -- Noble Judicator's Chestguard / Heroic Judicator's Chestguard
	AddItemPair(34386, 34170); -- Pantaloons of Growing Strife / Pantaloons of Calming Strife
	AddItemPair(34388, 34192); -- Pauldrons of Berserking / Pauldrons of Perseverance
	AddItemPair(34399, 34233); -- Robes of Ghostly Hatred / Robes of Faltered Light
	AddItemPair(34393, 34202); -- Shoulderpads of Knowledge's Pursuit / Shawl of Wonderment
	AddItemPair(34402, 34332); -- Shroud of Chieftain Ner'zhul / Cowl of Gul'dan
	AddItemPair(34391, 34209); -- Spaulders of Devestation / Spaulders of Reclamation
	AddItemPair(34389, 34193); -- Spaulders of the Thalassian Defender / Spaulders of the Thalassian Savior
	AddItemPair(34407, 34351); -- Tranquil Moonlight Wraps / Tranquil Majesty Wraps
	AddItemPair(34398, 34212); -- Utopian Tunic of Elun / Sunglow Vest

	-- == SUNWELL T6 PIECES == --
	-- Belt of the Forgotten Conqueror (Paladin, Priest, Warlock)
	AddItemList(34853, {34527, 34541, 34528, 34487, 34485,
	                    34488, 33882, 33888,
						33900, 33903, 33909});

	-- Belt of the Forgotten Protector (Warrior, Hunter, Shaman)
	AddItemList(34854, {34549, 34546, 34547, 34543, 34542, 34545, 33877, 33895, 33898, 33811, 33907});

	-- Belt of the Forgotten Vanquisher (Rogue, Mage, Druid)
	AddItemList(34855, {34557, 34558, 34554, 34555, 34556,
	                    33879, 33885, 33891,
						33912, 33915});

	-- Boots of the Forgotten Conqueror (Paladin, Priest, Warlock)
	AddItemList(34856, {34562, 34564, 34561, 34560, 34559,
	                    34563, 33884, 33890,
						33902, 33905, 33911});

	-- Boots of the Forgotten Protector (Warrior, Hunter, Shaman)
	AddItemList(34857, {34570, 34568, 34569, 34565, 34567, 34566, 33878, 33896, 33899, 33812, 33908});

	-- Boots of the Forgotten Vanquisher (Rogue, Mage, Druid)
	AddItemList(34858, {34574, 34575, 34571, 34572, 34573, 33880, 33886, 33892, 33914, 33916});

	-- Bracers of the Forgotten Conqueror (Paladin, Priest, Warlock)
	AddItemList(34848, {34434, 34436, 34435, 34431, 34432, 34433, 33883, 33889, 33901, 33904, 33910});

	-- Bracers of the Forgotten Protector (Warrior, Hunter, Shaman)
	AddItemList(34851, {34443, 34441, 34442, 34437, 34438, 34439, 33876, 33894, 33897, 33813, 33906});

	-- Bracers of the Forgotten Vanquisher (Rogue, Mage, Druid)
	AddItemList(34852, {34447, 34448, 34446, 34445, 34444, 33881, 33887, 33893, 33913, 33917});


	THLM_PopulateExtendedTable = THLM_NullFunc;
end

function THLM_PopulateExtendedInfo(item)
	THLM_PopulateExtendedTable();

    local _, _, itemnum = string.find(item, "item:(%d+):%d+:%d+:%d+");
	if not itemnum then
		THLM_HideAuctionExtendedSection();
		return;
	end

	local shortitem = "item:" .. itemnum;
	if THLM_ItemsWithExtendedInfo[shortitem] then
		local extendedItemNum = 0 + THLM_ItemsWithExtendedInfo[shortitem];
		THLM_ShowAuctionExtendedSection(extendedItemNum);
	else
		THLM_HideAuctionExtendedSection();
	end
end

function THLM_HideAuctionExtendedSection()
	loadingExtendedDataItem = nil;

	THLM_AuctionFrameExtendedFrame:Hide();
	THLM_AuctionFrameExtendedSlotTexture:Hide();
	THLM_AuctionFrameExtendedText:Hide();
	THLM_AuctionFrameExtendedIconFrame2:Hide();
	THLM_AuctionFrame:SetHeight(84);
end

local loadingExtendedDataItem = nil;

function THLM_ShowAuctionExtendedSection(itemid)
    local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType,
          itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(itemid);

	if (itemName) then
		loadingExtendedDataItem = nil;

	    SetItemButtonTexture(THLM_AuctionFrameExtendedIconFrame2, itemTexture);

        local _, _, _, itemColor = GetItemQualityColor(itemRarity);
		THLM_AuctionFrameExtendedText:SetText(itemColor .. itemName);

		THLM_AuctionFrameExtendedIconFrame2.itemid = itemid;
		THLM_AuctionFrameExtendedIconFrame2.itemlink = itemColor .. "\124Hitem:" .. itemid .. ":0:0:0:0:0:0:0\124h[" .. itemName .. "]\124h\124r";
		THLM_AuctionFrameExtendedText:Show();
		THLM_AuctionFrameExtendedIconFrame2:Show();
		THLM_SetUsableColor(THLM_AuctionFrameExtendedIconFrame2, "item:" .. itemid);
	else
		if (not loadingExtendedDataItem) or (not (itemid == loadingExtendedDataItem)) then
			THLM_ScanningTooltip:SetHyperlink("item:" .. itemid);
		end
		loadingExtendedDataItem = itemid;
	end
	THLM_AuctionFrame:SetHeight(104);
	THLM_AuctionFrameDecoration:Hide();
	THLM_AuctionFrameExtendedFrame:Show();
	THLM_AuctionFrameExtendedSlotTexture:Show();
end

function THLM_SetUsableColor(frame, item)
	local i;
	local isUsable = 1;
	THLM_ScanningTooltip:ClearLines();
	THLM_ScanningTooltip:SetHyperlink(item);
	for i = 1, THLM_ScanningTooltip:NumLines() do
		local textitem = getglobal("THLM_ScanningTooltipTextLeft" .. i);
		if (textitem) then
			local textstr = textitem:GetText();
			local r, g, b, a = textitem:GetTextColor();
			if (textstr) then
				local redstr = "";
				if (r + 0.1 > 1.0) and (g - 0.126 < 0.0) and (b - 0.126 < 0.0) then
					isUsable = nil;
				end
				--ChatFrame1:AddMessage( "left "..i..": ".. textitem:GetText() );
				--ChatFrame1:AddMessage( "r="..r.." g="..g.." b="..b.." a="..a);
			end
		end
		textitem = getglobal("THLM_ScanningTooltipTextRight" .. i);
		if (textitem) then
			local textstr = textitem:GetText();
			local r, g, b, a = textitem:GetTextColor();
			if (textstr) then
				local redstr = "";
				if (r + 0.1 > 1.0) and (g - 0.126 < 0.0) and (b - 0.126 < 0.0) then
					isUsable = nil;
				end
				--ChatFrame1:AddMessage( "right "..i..": ".. textitem:GetText() );
				--ChatFrame1:AddMessage( "r="..r.." g="..g.." b="..b.." a="..a);
			end
		end
	end

    --local isUsable, noMana = IsUsableItem(item);
	if (not isUsable) then
		--SetItemButtonNameFrameVertexColor(frame, 1.0, 0, 0);
		--SetItemButtonSlotVertexColor(frame, 1.0, 0, 0);
		SetItemButtonTextureVertexColor(frame, 0.9, 0, 0);
		SetItemButtonNormalTextureVertexColor(frame, 0.9, 0, 0);
	else
		--SetItemButtonNameFrameVertexColor(frame, 0,5, 0.5, 0.5);
		--SetItemButtonSlotVertexColor(frame, 1.0, 1.0, 1.0);
		SetItemButtonTextureVertexColor(frame, 1.0, 1.0, 1.0);
		SetItemButtonNormalTextureVertexColor(frame, 1.0, 1.0, 1.0);
	end
end

function THLM_EndAuction()
    THLM_CurrentItem = nil;
    THLM_CurrentExpireTime = 0;
    THLM_CurrentAuctioneer = nil;
    THLM_AuctionFrame:Hide();
    StaticPopup_Hide("THLM_PLACE_BID");
end

function THLM_CancelCurrentAuction()
    THLM_EndAuction();
end

function THLM_GetRaidRank(nick)
    local rank;
    local numRaidMembers = GetNumRaidMembers();
    local i;
    for i = 1, numRaidMembers do
        local uname = UnitName("raid"..i);
        if (uname and uname == nick) then
            _, rank = GetRaidRosterInfo(i);
        end
    end
    if not rank then
       rank = 0;
    end
    return rank;
end

function THLM_ParseRaidChatMessage(msg, author)
    local mpos = string.find(msg, THLM_AUCTIONSTART_SUFFIX);
    if (mpos and THLM_GetRaidRank(author) >= 1) then
        local linktext = string.sub(msg, 0, mpos-1);
        local _, _, itemid = string.find(linktext,"(item:%d+:%d+:%d+:%d+)");
        if itemid then
            THLM_ShowAuctionWindow(itemid, linktext, author);
        else
            THLM_Print("Could not find item link");
        end
    end

    local _, _, ilink = string.find(msg, THLM_AUCTIONCANCEL_PREFIX.."(.+)"..THLM_AUCTIONCANCEL_SUFFIX);
    if (ilink and author==THLM_CurrentAuctioneer) then
        THLM_CancelCurrentAuction();
    end
end

function THLM_strsplit(delimiter, text)
  local list = {}
  local pos = 1
  if strfind("", delimiter, 1) then -- this would result in endless loops
    error("delimiter matches empty string!")
  end
  while 1 do
    local first, last = strfind(text, delimiter, pos)
    if first then -- found?
      tinsert(list, strsub(text, pos, first-1))
      pos = last+1
    else
      tinsert(list, strsub(text, pos))
      break
    end
  end
  return list
end

local THLM_LastPointMessages = {};

function THLM_ProcessPointMessage(msg, funcname, alltab)
    THLM_LastPointMessages[funcname] = {
		msg = msg,
		time = GetTime()
	};
	THLM_DebugPrint("processing point message " .. msg);

    local mm = THLM_strsplit("+", msg);
    if not mm then
        return;
    end
    local i;
    for i = 1, getn(mm) do
        --THLM_DebugPrint("processing "..mm[i].." for "..funcname.."...");
        local _, _, thisName, thisValue = string.find(mm[i], "(%a+)(-?%d+)");
        if alltab=="ap" then
            THLM_AllPoints[thisName] = thisValue;
        elseif alltab=="amin" then
            THLM_AllMinBids[thisName] = thisValue;
        elseif alltab=="amax" then
            THLM_AllMaxBids[thisName] = thisValue;
        end
        if (thisName) then
            if (thisName == UnitName("player")) then
                local f = getglobal(funcname);
                f(thisValue + 0);
            end
        end
    end
end

function THLM_DisplayLastPointMessages()
    THLM_Print("Last point messages received:");
	table.foreach(THLM_LastPointMessages, function(k,v)
		local secAgo = math.ceil(GetTime() - v.time);
		THLM_Print(k .. "='" .. v.msg .. "' (" .. secAgo .. " sec ago)");
	end);
	THLM_Print("Done.");
end

function THLM_ScanForFiles()
	local needRestart = nil;

	-- THLootMod2.xml
	if ((not THLM_FILE_LOOTMOD2_XML)
	-- THLootMod2.lua
		or (not THLM_FILE_LOOTMOD2_LUA)
	-- TH_CTRAChannelHook.lua
		or (not THCH_Global_LoadedVersion)
	-- THLootMod2-invite.xml
		or (not THLM_FILE_LOOTMOD2_INVITE_XML)
	-- THLootMod2-invite.lua
		or (not THLM_FILE_LOOTMOD2_INVITE_LUA)
	-- THLootMod2-lootchart.xml
		or (not THLM_FILE_LOOTMOD2_LOOTCHART_XML)
	-- THLootMod2-lootchart.lua
		or (not THLM_FILE_LOOTMOD2_LOOTCHART_LUA)
	-- THLootMod2-server.xml
		or (not THLM_FILE_LOOTMOD2_SERVER_XML)
	-- THLootMod2-server.lua
		or (not THLM_FILE_LOOTMOD2_SERVER_LUA)
	-- THLootMod2-chat.xml
		or (not THLM_FILE_LOOTMOD2_CHAT_XML)
	-- THLootMod2-chat.lua
		or (not THLM_FILE_LOOTMOD2_CHAT_LUA)

		) then
		needRestart = 1;
	end

	if (needRestart) then
		StaticPopupDialogs["THLM_NEED_RESTART"] = {
			text = "You must restart World of Warcraft completely to fully install your new version of THLootMod2. Simply performing " ..
				"a reloadui is not sufficient. You may receive error messages if you do not restart now.",
			button1 = "Quit to Desktop",
			button2 = "Later",
			OnAccept = function()
				Quit();
			end,
			timeout = 0,
			whileDead = 1,
			hideOnEscape = 0
		};
		StaticPopup_Show("THLM_NEED_RESTART", "", "");
	end

	-- Free the memory used by this and ensure it doesn't get called a second time.
	THLM_ScanForFiles = THLM_NullFunc;
end

function THLM_NullFunc()
end

StaticPopupDialogs["THLM_NEED_UPGRADE"] = {
        text = "Your copy of THLootMod2 is out of date. Please install the newest version.", -- overridden before shown
        button1 = TEXT(ACCEPT),
        OnAccept = function()
			-- Gets overridden before being shown
        end,
        timeout = 0,
        whileDead = 1,
        hideOnEscape = 1
}

function THLM_CompareVersion(cver)
	if not cver then
		return;
	end
	if THLM_VersionWarningShown and THLM_VersionWarningShown >= cver then
		return;
	end
	if cver > THLM_VERSION then
		StaticPopupDialogs["THLM_NEED_UPGRADE"].text = "Your copy of THLootMod2 is out of date. The latest is version "..cver..". You have version " ..
			THLM_VERSION .. ". " ..
			"Please install the newest version.";
		StaticPopupDialogs["THLM_NEED_UPGRADE"].OnAccept = function()
			THLM_VersionWarningShown = cver;
		end;
		StaticPopup_Show("THLM_NEED_UPGRADE", "", "");
	end
end

function THLM_CTMessage(nick, msg)
    local _, _, mtype, payload = string.find(msg, "^_THLM2?_(..) ?(.*)");
    if payload==nil then
        payload = "";
    end
    if (mtype=="PT") then
        --THLM_DebugPrint("processing PT");
        THLM_ProcessPointMessage(payload, "THLM_SetMyPoints", "ap");
    elseif (mtype=="MB") then
        --THLM_DebugPrint("processing MB");
        THLM_ProcessPointMessage(payload, "THLM_SetMyMinBid", "amin");
    elseif (mtype=="XB") then
        --THLM_DebugPrint("processing XB");
        THLM_ProcessPointMessage(payload, "THLM_SetMyMaxBid", "amax");
    elseif (mtype=="VR") then
        THLM_CompareVersion(tonumber(payload));
	elseif (mtype=="SA") then
		THLM_HandleAddonSpamAlert(payload, nick);
    else
        --THLM_DebugPrint("Unknown messge type="..mtype.." payload="..payload);
        --THLM_Print("UPGRADE YOUR COPY OF THLOOTMOD2");
    end
end

function THLM_OnUpdate()
    if (THLM_CurrentExpireTime > 0) then
        local now = GetTime();
        if ( now >= THLM_CurrentExpireTime ) then
            THLM_EndAuction();
        else
            local remain = THLM_CurrentExpireTime - now;
            THLM_AuctionFrameTimer:SetMinMaxValues(0, THLM_AUCTION_TIME);
            THLM_AuctionFrameTimer:SetValue(remain);
            if THLM_NeedIcon then
				THLM_PopulateAuctionWindow(THLM_NeedIcon, THLM_NeedIconItemLink);
            end
        end
    end
	if loadingExtendedDataItem then
		THLM_ShowAuctionExtendedSection(loadingExtendedDataItem);
	end
end

function THLM_OnEvent(event)
    if (event == "CHAT_MSG_RAID" or event == "CHAT_MSG_RAID_LEADER") then
        THLM_ParseRaidChatMessage(arg1, arg2);
    elseif (event == "CHAT_MSG_WHISPER") then
		if THLM_Debug then
        	THLM_ParseRaidChatMessage(arg1, arg2);
		end
		THLM_CheckForMainWhisper();
		THLM_CheckForKeywordWhisper(arg1, arg2);
    elseif (event == "PLAYER_LOGIN") then
        THLM_SetupChatFrameHook();
	elseif (event == "CHAT_MSG_CHANNEL") then
		THLM_CheckForSpamMessage();
		THLM_ScanForFiles();
	elseif (event == "PARTY_INVITE_REQUEST") then
		THLM_MaybeAcceptGroupInvite(arg1);
    end
end

function THLM_OnLoad()
    this:RegisterEvent("PLAYER_LOGIN");
    this:RegisterEvent("CHAT_MSG_RAID");
    this:RegisterEvent("CHAT_MSG_RAID_LEADER");
    --if THLM_Debug then
    --    this:RegisterEvent("CHAT_MSG_WHISPER");
    --end
	this:RegisterEvent("CHAT_MSG_CHANNEL");
	this:RegisterEvent("CHAT_MSG_WHISPER");
	this:RegisterEvent("PARTY_INVITE_REQUEST");

    THCH_RegisterCTRAChannelMessage("_THLM_PT", THLM_CTMessage);
    THCH_RegisterCTRAChannelMessage("_THLM_MB", THLM_CTMessage);
    THCH_RegisterCTRAChannelMessage("_THLM_XB", THLM_CTMessage);
    THCH_RegisterCTRAChannelMessage("_THLM_VR", THLM_CTMessage);
    THCH_RegisterCTRAChannelMessage("_THLM2_SA", THLM_CTMessage);

	SLASH_THPOINTDEBUG1 = "/thpointdebug";
	SlashCmdList["THPOINTDEBUG"] = function(msg)
		THLM_DisplayLastPointMessages();
	end;
end
