--
--  TH_CTRAChannelHook
--
--  You can drop this .lua file into your AddOn's directory, include it
--  in your toc file, and it will just work. If other addons are using
--  their own copies of TH_CTRAChannelHook.lua, collectively they will only
--  hook the channel once, so no extra time is wasted.
--
--  It is recommended you prefix your message keys with "_" and your addon
--  name to possible collision with other addons, and CTRA itself.
--
--  Be sure to add CT_RaidAssist as a dependency or an optional dependency
--  to your addon.
--
--  API:
--     THCH_RegisterCTRAChannelMessage(msgcode, func)
--         Installs a hook for messages that begin with the value of
--         the "msgcode" parameter. The function "func" will be called
--         whenever a message that begins with this code is received.
--
--  Example usage:
--
--     function MyAddon_SampleFunc(nick, msg)
--         -- Do something here!
--     end
--
--     function MyAddon_OnLoad()
--         THCH_RegisterCTRAChannelMessage("_MyAddonSF", MyAddon_SampleFunc);
--     end


local THCH_VERSION = 3;

if (not THCH_Global_LoadedVersion or THCH_VERSION > THCH_Global_LoadedVersion) then
    THCH_Global_LoadedVersion = THCH_VERSION;

    THCH__Initialize = function()
        THCH_Global_LoadedVersion = 99999;
        THCH_Hooks = {};

        local hookFrame = CreateFrame("Frame", nil);
        hookFrame:SetScript("OnEvent", function() THCH__ProcessMessage(event); end);
        hookFrame:RegisterEvent("CHAT_MSG_ADDON");
        hookFrame:Show();

        THCH__Initialize = function() end;
    end;

    THCH__ProcessMessage = function(event)
       if (event == "CHAT_MSG_ADDON" and arg1 == "THCH") then
          local message = arg2;
          local dist = arg3;
          local nick = arg4;
          local k, v;
          for k,v in pairs(THCH_Hooks) do
             if (string.find(message, k, 1, true)) then
                return v(nick, message, dist);
             end
          end
       end
    end;

    THCH_SendMessage = function(message, dist)
        THCH__Initialize();
        if not dist then
            dist = "RAID";
        end
        SendAddonMessage("THCH", message, dist);
    end;

	THCH_SendWhisper = function(message, target)
		THCH__Initialize();
		if not target then
			return;
		end
		SendAddonMessage("THCH", message, "WHISPER", target);
	end;

    THCH_RegisterCTRAChannelMessage = function(msgcode, func)
        THCH__Initialize();
        THCH_RegisterCommChannelMessage(msgcode, func);
    end;

    THCH_RegisterCommChannelMessage = function(msgcode, func)
        THCH__Initialize();
        if (type(msgcode) == "string") and (type(func)=="function") then
            THCH_Hooks[msgcode] = func;
        end
    end;
end
